#!/usr/bin/perl

# tuc2eqsl.pl
#
# Reformat Tucnak ADIF output specifically for eQSL.cc 
# Concatenate Sent RST and QSO number, (59/567)
# remove extraneous info, add a free-form comment. 
#
# Copyright  2009, 2010, 2015 Dave Gilligan. <g1ogy*_\@_*g1ogy.com>

#############################################################################
# This program is free software; you can redistribute it and/or modify it   
# under the terms of the GNU General Public License as published by the 
# Free Software Foundation; either version 3 of the License, or (at your    
# option) any later version. 
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
# Public License for more details. 
#	http://www.gnu.org/licenses/gpl.html
#
#     In a word: If it breaks - you get to keep all the pieces.
#
#############################################################################


use Getopt::Long;

# Banner headers
my $prog = "tuc2eQSL";
my $version = "RC1.01";
my $datestring="[20100224]";
my $auth = "Author: Dave Gilligan <g1ogy\@g1ogy.com> ";
my $arights="Licenced under GNU GPL.";


GetOptions('c|comment=s', 's|stripheader', 'h|help');


# define Usage

    die "\n\"$prog\"  $version       $datestring     $auth
                                                        $arights
  Usage:
  
	\"$prog\" [-c|comment \"<Comment Text>\"] [-h|help] \\ 
  		   [-s|stripheader] <Input-File> [<Output-File>]
  		
        \"$prog\" reformats Tucnak ADIF output to concatenate RST and QSO
	number - Contest-style (ie: 57/246) and provide the facility 
	to add a free-form comment for the bottom of the eQSL Card. 
        Comment most useful to describe the specific contest and add 
	/P site locator when group uses multiple sites.
            
        \"$prog\" requires an INPUT file name argument and 
        optionally, an OUTPUT file name.
        Note that the output is printed to the screen AS WELL AS the
	output file (default or user-entered) to provide a visual check.
	Banner & Credits are printed only to the screen but can be
	suppressed (with -s) if you are building a pipe.

        NOTE:  This program does not cope with embedded \"Remarks\"
        [\`# + Text\` on the input line]. Edit Tucnak ADIF file to remove. 
             
        Please see the README file distributed with this program or
        execute  \"$prog\" -[h]elp to display the README here.

    Evidently, this program expects to be run under Perl on Linux (or other 
    Un*x-style O/S) or a 32-bit (or later) version of MS Windows. \n"
    
    unless @ARGV;
    
    
    # Print Banner
	    unless ($opt_s)     {
	        Banner();
	    }

# define local variables

    my ($staticComment) =	$opt_c;
    my ($ADIFin)	=	$ARGV[0];
    my ($ADIFout) 	= 	$ARGV[1];
    my ($qsl_Comment)	=	"";
    my ($OUTPUTf)	=	"";
    my (@logBook);

###
   
# main  

    if ($ARGV[1] ne "")  {
        open (ADIFOUT,">$ADIFout") || die "\nCannot Create Output file: \"$ADIFout\"!\n";
		$OUTPUTf = $ADIFout;	
        }
        else  { 
            open (ADIFOUT,">$ADIFin.proc.adif") || die "\nCannot Create Output file: \"$ADIFin.proc.adif\"!\n"; 
		$OUTPUTf = "$ADIFin.proc.adif";	
            }
    open (ADIFIN, "$ADIFin") || die "\nInput File - \"$ADIFin\" - not Found\! Check the path : \n$!";

   
    while (<ADIFIN>) {
	next unless /^<BAND/;		# skip the header
	# Not needed - stops on EOF anyhow! 
    # last unless /^</;		    # stop on a blank line

        chomp;
        
        # get and assign each element of the input line (source: Tucnak :: Export as ADIF) 

        	
		(	$band,
			$qso_date,    
			$time_on,            
			$call,        
			$mode,
			$rst_sent,    
			$no_sent,
			$stx,         
			$rst_rcvd,    
			$srx,    
			$no_rcvd,
			$gridsquare,     
			$qrb,         
			$qtf,
			$points,
			$operator,    
			$prop_mode,  
			$eor    ) = split;


# Process the raw QSO line before reducing the dataset
  		reportConcat();


# Update the pre-defined Comment var

  		addComment();

# Reduce the dataset to only the fields required at eQSL by selecting fields 
# from the input and writing to an array as a space-delimited record.
# Suppress "ERROR" QSOs. eQSL barfs otherwise


	unless ($call eq "<CALL:5>ERROR")	{

		push (@logBook, (	
					$band." ".
					$qso_date." ".
					$time_on." ".        
					$call." ".    
					$mode." ".
					$rst_sent." ".
					$prop_mode." ".
					$qsl_Comment
					)); 
				}
# finished
	}


# ...so write the output file  			

# ADIF Header
	print ADIFOUT "Contest logged with Tucnak v4\n";
	print ADIFOUT "eQSL data processed by tuc2eQSL.pl by G1OGY\n";
	print ADIFOUT "<PROGRAMID:6>Tucnak <ADIF_VER:5>2.2.0  <EOH>\n";

		foreach $qsoRecord (@logBook)	{
			print  $qsoRecord ." <EOR>\n" ;
			print ADIFOUT $qsoRecord ." <EOR>\n" ;
			}

#  Print notifcations
	print "\n\n\n 	Output in $OUTPUTf\n\n\n";

	    unless ($opt_s)     {
	        Credits();
	    }

# Cleanup
	close (ADIFIN); close (ADIFOUT);

# subRoutines
   ############################################################################################

sub addComment	{
	# Replace the (empty) comment value with the text from the command line
	
  		if ($opt_c)  {
			$qsl_Comment = "<QSL_COMMENT:" . length($staticComment) . ">" . $staticComment;
  			}
	}

   ############################################################################################
 
sub reportConcat  {
	# concat the Sent report with the sent serial - 59/456
	# get the stx var and strip off the <ADIF:##> tag

	# Yes, well...
	# get the length of rst_sent (could be 2 or 3)
	# get the sent report value from rst_sent
	# get the serial no from stx - get it's length (3 or 4)
	# add the initial length of rst_sent to the length of the serial no. plus the seperator
	
	
	$sentSerialOrdinal=substr $rst_sent, 10, 1;
	$sentRpt=substr $rst_sent, 12, $sentSerialOrdinal;
	$sentSerial=substr $stx, 7;
	$sentSerLen=length($sentSerial);
	$sentRptLen=$sentSerialOrdinal + $sentSerLen +1;
	$rst_sent = substr $rst_sent, 0, 10, . $sentRptLen .">". $sentRpt ."/". $sentSerial;
	}

    ############################################################################################
    # Banner
    
    sub Banner  {
        print ("\n  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n");
        print ("\n\"$prog\"  $version  $datestring  $auth\n   $arights\n");
        print ("\n  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n");
        }
    
    ############################################################################################
    # Credits
    
    sub Credits {
        
        print ("\n  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n");
        print ("    Credits: ");
        print ("\n      \"$prog\" is written against \"Perl v5.10\" and compiled \n");
        print ("        - where relevant - with \"Perl2exe\" by Indigostar Software.\n\n"); 
        print ("        \"$prog\": GNU General Public Licence applies.\n");
        print ("\n  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n");
        }
    
    ############################################################################################
    # Help

    ############################################################################################

__END__
    
